@@ -10,7 +10,9 @@ import win32con
 import win32gui
 import win32ui
 
-import utils
+from . import img_data_folder
+from . import utils
+from . import exception
 
 class BITMAPINFOHEADER(Structure):
     _fields_ = [
@@ -28,22 +30,18 @@ class BITMAPINFOHEADER(Structure):
     ]
 
 
-def screenshotEx(window_name : str = None
-                , rect : tuple = None) -> np.ndarray:
-    target_window = win32con.NULL
+def screenshot(window_handle : int, rect : tuple = None, direct_view : bool = False) -> np.ndarray:
+    target_handle = win32gui.GetDesktopWindow() if direct_view else window_handle
 
-    if window_name:
-        target_window = win32gui.FindWindow(win32con.NULL, window_name)
-        left, top, right, bottom = rect or win32gui.GetClientRect(target_window)
-        width = right - left
-        height = bottom - top
-    else:
-        left, top = 0, 0
-        width = win32api.GetSystemMetrics(win32con.SM_CXSCREEN)
-        height = win32api.GetSystemMetrics(win32con.SM_CYSCREEN)
+    left, top, right, bottom = rect or win32gui.GetClientRect(window_handle)
+    if direct_view:
+        left, top = win32gui.ClientToScreen(window_handle, (left, top))
+        right, bottom = win32gui.ClientToScreen(window_handle, (right, bottom))
 
-    
-    targetDC = win32gui.GetDC(target_window)
+    width = right - left
+    height = bottom - top
+
+    targetDC = win32gui.GetDC(target_handle)
     compatibleDC = win32gui.CreateCompatibleDC(targetDC)
 
     win32gui.SetStretchBltMode(compatibleDC, win32con.COLORONCOLOR)
@@ -70,7 +68,8 @@ def screenshotEx(window_name : str = None
     win32gui.BitBlt(compatibleDC, 0, 0, width, height, targetDC, left, top, win32con.SRCCOPY)
 
     windll.gdi32.GetDIBits(compatibleDC, bitmap.handle, 0, height,
-                     cast(img.ctypes.data, POINTER(c_byte)),
+                     #cast(img.ctypes.data, POINTER(c_byte)),
+                     img.ctypes.data_as(POINTER(c_byte)),
                      byref(bitmapInfo),
                      win32con.DIB_RGB_COLORS)
 
@@ -79,6 +78,20 @@ def screenshotEx(window_name : str = None
     win32gui.ReleaseDC(win32con.NULL, targetDC)
 
     return cv2.cvtColor(img, cv2.COLOR_BGRA2BGR) if img.shape[-1] == 4 else img
+    
+
+def desktop_screenshot(rect : tuple = None) -> np.ndarray:
+    desktop_window = win32gui.GetDesktopWindow()
+    return screenshot(desktop_window, rect)
+
+
+def screenshotEx(window_name : str, rect : tuple = None) -> np.ndarray:
+    window = win32gui.FindWindow(win32con.NULL, window_name)
+    if not window:
+        raise exception.HandleException("윈도우 핸들을 구하지 못했습니다")
+
+    return screenshot(window, rect)
+
 
 class IMAGE(Structure):
     _pack_ = 16
@@ -90,7 +103,7 @@ class IMAGE(Structure):
 
 class imageUtil:
     def __init__(self):
-        dll_path = os.path.join(os.environ["DLL_FOLDER"], "ImageDLL.dll")
+        dll_path = "ImageDLL.dll"
         self.dll = windll.LoadLibrary(dll_path)
         self._ImageSearchEx = self.dll["ImageSearchEx"]
         self._ImageSearchEx.argtypes = (wintypes.LPCWSTR, wintypes.LPCWSTR, POINTER(wintypes.RECT), wintypes.DWORD)
@@ -106,26 +119,26 @@ class imageUtil:
         self._ImageSearchEx_Raw.argtypes = (POINTER(IMAGE), POINTER(IMAGE), POINTER(wintypes.RECT), wintypes.DWORD)
         self._ImageSearchEx_Raw.restype = wintypes.INT
 
-        self.ImageSearchEx_Raw_All = self.dll["ImageSearchEx_Raw_All"]
-        self.ImageSearchEx_Raw_All.argtypes = (POINTER(IMAGE), POINTER(IMAGE), POINTER(wintypes.RECT), wintypes.UINT, wintypes.DWORD)
-        self.ImageSearchEx_Raw_All.restype = wintypes.INT
+        self._ImageSearchEx_Raw_All = self.dll["ImageSearchEx_Raw_All"]
+        self._ImageSearchEx_Raw_All.argtypes = (POINTER(IMAGE), POINTER(IMAGE), POINTER(wintypes.RECT), wintypes.UINT, wintypes.DWORD)
+        self._ImageSearchEx_Raw_All.restype = wintypes.INT
 
         self.find_rects_len = 100
         self.find_rects = (wintypes.RECT * self.find_rects_len)()
 
     def __del__(self):
-
-        FreeLibrary = windll.kernel32["FreeLibrary"]
+        pass
+        """FreeLibrary = windll.kernel32["FreeLibrary"]
         FreeLibrary.argtypes = (wintypes.HMODULE,)
         FreeLibrary.restype = c_int32
-        FreeLibrary(self.dll._handle)
+        FreeLibrary(self.dll._handle)"""
 
     def imageSearchEx(self, src : np.ndarray, temp : np.ndarray, except_color : tuple = (255, 0, 0), find_all : bool = True) -> list:
         src_data = IMAGE(src.ctypes.data_as(POINTER(c_ubyte)), *src.shape)
         temp_data = IMAGE(temp.ctypes.data_as(POINTER(c_ubyte)), *temp.shape)
 
         if find_all:
-            ret = self.ImageSearchEx_Raw_All(byref(src_data), byref(temp_data)
+            ret = self._ImageSearchEx_Raw_All(byref(src_data), byref(temp_data)
                                         , cast(pointer(self.find_rects), POINTER(wintypes.RECT)), self.find_rects_len, win32api.RGB(*except_color))
             
             self.find_rects = self.find_rects
@@ -148,14 +161,9 @@ def imageSearchEx(src : np.ndarray, temp : np.ndarray, except_color : tuple = (2
 
 
 def cv2_imread(img_path : str, flag : int = cv2.IMREAD_UNCHANGED) -> np.ndarray:
-    try:
-        data = np.fromfile(img_path, np.uint8)
-        img = cv2.imdecode(data, flag)
-        img = cv2.cvtColor(img, cv2.COLOR_BGRA2BGR) if img.shape[-1] == 4 else img
-
-        return img
-    except Exception as e:
-        return None
+    data = np.fromfile(img_path, np.uint8)
+    img = cv2.imdecode(data, flag)
+    return img
 
 def cv2_imreads(img_folder_path : str, flag : int = cv2.IMREAD_UNCHANGED) -> dict:
     img_paths = utils.get_image_file_list(os.path.join(img_folder_path, "**", "*.*"))
@@ -179,7 +187,7 @@ def template_match(src : np.ndarray, temp : np.ndarray
     except_color : 예외 색상
     """
     
-    mask = ...
+    mask = None
     if except_color:
         mask = cv2.inRange(temp, except_color, except_color)
         mask = cv2.bitwise_not(mask)
